' Original Morse code routines based on original PICAXE CW beacon program by Steve Paige, VK6HV, (c)2008

' Modifications & additions by Robert, VA3ROM April 2023 (many PICAXE programming "tricks" used to maximize features
' Uses Elecno Snap Circuits ARCADE kit & replaces original ARCADE program code

'__________________________________________________MORSE CODE CHARACTERS___________________________________________________

'       A .-    B -...  C -.-.  D -..    E .      F ..-.
'       G --.   H ....  I ..    J .---   K -.-    L .-..
'       M --    N -.    O ---   P .--.   Q --.-   R .-.
'       S ...   T -     U ..-   V ...-   W .--    X -..-
'       Y -.--  Z --.. 

'       1 .---- 2 ..--- 3 ...-- 4 ....-  5 .....
'       6 -.... 7 --... 8 ---.. 9 ----.  0 -----

'       FULL STOP          [.]  .-.-.-     COLON                   [:]  ---...
'       COMMA              [,]  --..--     SEMICOLON               [;]  -.-.-.
'       QUESTION MARK      [?]  ..--..     EQUAL SIGN (DOUBLE DASH)[=]  -...-
'       APOSTROPHE         [']  .----.     PLUS                    [+]  .-.-.
'       EXCLAMATION MARK   [!]  -.-.--     HYPHEN minus            [-]  -....-
'       FWD SLASH          [/]  -..-.      UNDERSCORE              [_]  ..--.-
'       PARENTHESIS open   [(]  -.--.      QUOTATION MARK          ["]  .-..-.
'       PARENTHESIS closed [)]  -.--.-     DOLLAR SIGN             [$]  ...-..-
'       AMPERSAND          [&]  .-...      AT SIGN (COMMAT) [WR]   [@]  .--.-.

'________________________________________ENCODED MORSE CODE ALPHANUMERIC CHARACTER SET_____________________________________

'       A   6,   B  17,   C  21,   D   9,   E   2,    F  20
'       G  11,   H  16,   I   4,   J  30,   K  13,    L  18
'       M   7,   N   5,   O  15,   P  22,   Q  27,    R  10
'       S   8,   T   3,   U  12,   V  24,   W  14,    X  25
'       Y  29,   Z  19 

'       1  62,   2  60,   3  56,   4  48,   5  32 
'       6  33,   7  35,   8  39,   9  47,   0  63

'       FULL STOP          	[.]  106     COLON                      [:]  71
'       COMMA              	[,]  115     SEMICOLON                  [;]  85
'       QUESTION MARK      	[?]  76      EQUAL SIGN (BREAK)	    [=]  49
'       APOSTROPHE         	[']  94      PLUS                       [+]  42
'       EXCLAMATION MARK   	[!]  117     HYPHEN (MINUS SIGN)        [-]  97
'       FORWARD SLASH      	[/]  41      UNDERSCORE                 [_]  108
'       PARENTHESIS (OPEN)   	[(]  45      QUOTATION MARK             ["]  82
'	  PARENTHESIS (CLOSED) 	[)]  109     DOLLAR SIGN          	    [$]  200
' 	  AMPERSAND          	[&]  34      AT (@) SIGN PROSIGN BAR WR [@]  86		SPACE [ ] 0
'__________________________________________________________________________________________________________________________
	
' Note: The PICAXE LED MC (U29) is requires > 3.0 volts to operate poperly because of the 7-segment LED displays. You can use 
' three rechargeable AA cells (3 x 1.25 or 3.75 volts) so long as you keep this in mind. AA alkaline cells are 3 x 1.5 or 4.5 volts

#no_data						'don't clear EEPROM current values
#terminal 4800					'denable PICAXE Editor serial terminal output at 4800 baud

'disconnects/disables serial input (serin) programming & enable as input port C.5 (pin 2) for Selector (S8) input
'to reprogram PICAXE chip ensure its power is disconnected/turned off, start download/programming process using connected cable
'reconnect by turning power on
disconnect						

'__________________________________________________________________________________________________________________________
' change these global constants as req'd
'__________________________________________________________________________________________________________________________
'Mersenne prime named after 17th century mathematician Marin Mersenne who studied them in early 17th century
'In mathematics, a Mersenne prime is a prime number that is one less than a power of two. That is, it is a prime number
'of the form Mn = 2n ? 1 for some integer n. Sometimes Mersenne numbers are defined to have the additional requirement
'that n must also be prime (51 are Merseene primes are known to exist & largest so far is 2^82,589,933 -1)
'A true "Mersenne Twister" pseudo random number generator (PRNG), created by Makoto Matsumoto, is difficult to
'implement on PICAXE MCUs but not impossible (requires using 32-bit words & based on Mersenne prime 2^19937 - 1)
'So the PICAXE uses an Xorshift PRNG, also called shift-register generators, are a class of pseudo random number
'generators created by mathematician & computer scientist George Marsaglia

'M13 = 2^13 - 1 = 8191 & M7 = 2^7 - 1 = 127 (13 & 7 are also prime numbers)
symbol PRIME_1 = 8191				'Mersenne primes: 3, 7, 31, 127, 8191 (rest > 65535 & PICAXE can't handle those)
symbol PRIME_2 = 127				

'adjust for differences in parts-per-million errors (all PICAXE chips have ppm errors in internal clock 
'plays speaker sounds are timed in 10 ms intervals so also need to decrease timing to compensate
'CODEX may be too fast for newbies so you may want to use the PARIS timing standard of 1200 ms
symbol TC = 1000					'1000 ms for CODEX for random number groups use 1200 ms or use for PARIS and plain language text

symbol TONE = 109					'default Morse code speaker tone used (change as req'd)

symbol WPM_MENU = 20				'default menu alphanumeric transmission speed (takes too long if user selects slow speeds)

'you may have to adjust these values slightly also to compensate for internal clock ppm timing errors
symbol DEBOUNCE = 50				'Selector (S8) key debounce delay in ms
symbol DELAY = 10000				'delay in ms before starting beacon transmission sequence (10000 ms = 10 s, change as req'd)

'__________________________________________________________________________________________________________________________
' Program macro code begins below
'__________________________________________________________________________________________________________________________

'__________________________________________________________________________________________________________________________
'for beacon mode edit lookup string (max. 256 characters) plus sign (+) marks end of transmission (EOT) prosign bar AR
'use to key real Morse code (CW) transmitter or send long plain language text strings for copy practice
'
'	Initialize: INDEX (points to next character in lookup table), CHARACTER (holds lookup table alphanumberic character)
'	Variables: INDEX, CHARACTER
'	Calls: Nothing
'	Returns: CHARACTER
'__________________________________________________________________________________________________________________________
#MACRO Beacon(INDEX, CHARACTER)

	'use "CODEX " transmission timing for random text groups to see how close actual word-per-minute (WPM) speed is to calculated speed 
	'adjust global constant "TC" as req'd to get WPM speed to +/- one WPM (PICAXE internal clocks all vary a bit)
	'use "PARIS " transmission time for conventional text sent in sentences without/without normal punctuation
	
	'lookup INDEX, ("CODEX CODEX CODEX CODEX CODEX CODEX CODEX CODEX CODEX CODEX CODEX CODEX CODEX CODEX CODEX CODEX CODEX CODEX CODEX CODEX +"), CHARACTER
	
	'lookup INDEX, ("PARIS PARIS PARIS PARIS PARIS PARIS PARIS PARIS PARIS PARIS PARIS PARIS PARIS PARIS PARIS PARIS PARIS PARIS PARIS PARIS +"), CHARACTER
	
	'on air beacon to be transmitted continously (+) plus sign marks end of text/transmission
	'use your Amateur Radio callsign & Maidenhead grid locator
	lookup INDEX, ("VVV DE CALLSIGN/B AA00AA +"), CHARACTER
	
#ENDMACRO

'__________________________________________________________________________________________________________________________
' Read Selector (S8) for key press: A (port C.5, pin 2), B (port C.5, pin 2 + port C.3, pin 4), C (port C.3, pin 4)

'	Initialize: VALUE (number value to check), MINI (mininum value of number), MAXI (maximum value of number)
'	Variables: SEED, SELECTOR, BTN_A, BTN_B, BTN_C, SPEAKER, TONE, VALUE, MINI, MAXI
'	Calls: Display_Menu, TX_Morse, In_Range
'	Returns: VALUE, SELECTOR
'	Notes: Circular buffer used so if VALUE is kept between MINI & MAXI values
'__________________________________________________________________________________________________________________________
#MACRO Read_Selector(VALUE, MINI, MAXI)

	do
		random SEED				'generate pseudo random number (not used but randomizes it more)

		'scramble PRNG see using system time & Mersenne prime number (not used but randomizes it even more)
		SEED = SEED XOR TIME XOR PRIME_2	
						
		SELECTOR = pins and 40		'check Selector (S8) for A, B or C keypress (A = 32, B = 40, C = 8)		
			pause DEBOUNCE		'debounce delay
		
	loop while SELECTOR = 0			'keep scanning Selector (S8) 0 = no button pressed
	
	if SELECTOR = BTN_A then		'Selector A button pressed, port C.5, pin 2 goes high
				
		inc VALUE				'increment menu choice & check in range of minimum & maximum values
			
		In_Range(VALUE, MINI, MAXI)
		
	elseif SELECTOR = BTN_B then		'Selector B button pressed (accept), port C.3, pin 4 & port C.5, pin 2 go high
			
		CHARACTER = ROGER			'send Morse code "R" for ROGER command acknowledged
			
		call TX_Morse
		
	elseif SELECTOR = BTN_C then		'Selector C button pressed, port C.3, pin 4 goes high  			
		  
		dec VALUE				'decrement menu choice & check in range of minimum & maximum values
					  
		In_Range(VALUE, MINI, MAXI)
		
	endif
 	 
#ENDMACRO

'__________________________________________________________________________________________________________________________
' macro function uses circular buffer to keep 1st 8-bit value in range between 2nd & 3rd 8-bit values (0 - 255)

'	Initialize: VALUE (number to check), MINI (mininum value of number), MAXI (maximum value of number)
'	Variables: VALUE, MINI, MAXI
'	Calls: Nothing
'	Returns: VALUE
'__________________________________________________________________________________________________________________________
#MACRO In_Range(VALUE, MINI, MAXI)

	if VALUE < MINI then			'wrap around if item value too low

		VALUE = MAXI
	
	elseif VALUE > MAXI then		'wrap around if item value too high
	
		VALUE = MINI

	endif

#ENDMACRO

'__________________________________________________________________________________________________________________________
' Macro function displays menu item encoded alphanumeric values on LED display & also transmits ASCII valuea in Morse code

'	Initialize: CHAR_A (1st encode char), CHAR_B (2nd encode char), CHAR_C (1st ASCII char), CHAR_D (2nd ASCII char)
'	Variables: CHAR_A, CHAR_B, CHAR_C, CHAR_D, ENCODED, CHARACTER
'	Calls: Display_Character, Tx_Morse
'	Returns: Nothing
'__________________________________________________________________________________________________________________________
#MACRO Display_Menu_Item(CHAR_A, CHAR_B, CHAR_C, CHAR_D)

 	DIT = TC / WPM_MENU			'send menu numbers @ default 20 wpm (in case user has set a really slow sending speed)
	
	ENCODED = CHAR_A				'transfer first encoded character to display
		call Display_Character		'display on rightmost LED display
	
	ENCODED = CHAR_B 				'transfer second encoded character to display
		call Display_Character		'shift first character to leftmost LED display & display this character on rightmost display
	
	CHARACTER = CHAR_C			'get 1st (leftmost) menu item character
		Call TX_Morse			'transmit in Morse code
	
	CHARACTER = CHAR_D			'get 2nd (rightmost menu item character
		Call Tx_Morse			'transit in Morse code
	
	DIT = TC / WPM				'restore user's wpm speed

#ENDMACRO

'__________________________________________________________________________________________________________________________
' Macro function writes/saves variable value to EEPROM address if value was changed by user (else nothing is changed)
' EEPROM writes do cuase wear & tear with a limited number of lifetime write cycleon

'	Initialize: FLAG (holds previous NUMBER value, NUMBER holds current value, INDEX point to EEPROM write address
'	Variables: FLAG, NUMBER, INDEX
'	Calls: Noting
'	Returns: Nothing
'__________________________________________________________________________________________________________________________
#MACRO Write_EEPROM(FLAG, NUMBER, INDEX)

	if FLAG <> NUMBER then			'has previous NUMBER value changed?

		write INDEX, NUMBER		'yes, so write/save new NUMBER to desired EEPROM memory address
	
	endif

#ENDMACRO

'__________________________________________________________________________________________________________________________
' Define global program variables & assign them to specific PICAXE 08M2+ memory locations
'__________________________________________________________________________________________________________________________

'byte or 8-bit variable declarations
symbol REPEATS = b0				'counter for number of five character code groups repeated
symbol GROUPS = b1				'total number of Morse code repeated groups (5 to 99) default = 20

symbol SEGMENT = b2				'specific LED segment to turn on/off
symbol SELECTOR = b3				'Selector (S8) button pressed value (button A = 32, B = 40 & C = 8)
symbol NUMBER = b4				'generic number variable/counter etc
symbol INDEX = b5 				'index pointer for alphanumeric encoded 7-segment LED lookup table

symbol FLAG = b6					'character bit even/odd & general purpose flag
symbol ENCODED = b7				'encoded Morse code or encoded character value

symbol WPM = b8					'Morse code words-per-minute sending speed (min = 5, max = 40 wpm)
symbol DIT = b9					'Morse code dit time in millisenconds (ms) = 1200 / WPM (default unadjusted)

symbol CHARACTER = b10				'alphanumeric CHARACTER value to display on 7-segment LED
symbol BIT_MASK = b11				'bit mask to check each bit in CHARACTER to determine LED segment to turn on/off

symbol DURATION = b12				'speaker tone, PTT & red LED on/off times

symbol ITEM = b13					'menu item index pointer & character lookup index alpha value

symbol MODULO = b14				'modulo value used to find alphanumric characters in lookup table
symbol OFFSET = b15				'offset used in lookup table with PRNG Morse code character generator

symbol DECIMAL = b16				'LED display decimal point in number req'd status flag

'word or 16-bit variable definitions
symbol DASH = w10					'dash interval in ms also used for interval between characters
symbol BCN_INTERVAL = w11			'interval time between beacons in milliseconds (1 to 60000 or 1 to 60 seconds)
symbol VPSU = w12					'PICAXE power supply DC voltage (VPSU) (5.5 to 1.8 VDC, brownout at 1.9 VDC)
symbol SEED = w13					'used by pseudo random number generator

'__________________________________________________________________________________________________________________________
' Define program global constants (these are fixed & CANNOT be changed during program execution)
'__________________________________________________________________________________________________________________________
symbol W_CLOCK = c.0				'clock pulse port C.0 (pin 7) shifts bits in C4094B shift-and-store bus registers

symbol S_DATA = c.1				'on/off port C.1. (pin 6) turns appropriate segment of 7-segment LED on/off 
symbol RED_LED = c.1				'red LED shares port with S_DATA (can also use as PTT port to drive transistor tx on/off switch)

symbol SPEAKER = c.2				'speaker output emulates what rotary spark gap Morse code sounded like

symbol PRESS_SWITCH = pinC.4			'press switch (hand key) port C.4 (pin 3) use pinC.4 when testing input value

symbol BTN_A = 32					'Selector (S8) button A pressed return value
symbol BTN_B = 40					'Selector (S8) button B pressed return value
symbol BTN_C = 8					'Selector (S8) button C pressed return value

SEED = PRIME_1					'PRNG starting seed

symbol NO = 0					'LED display decimal point not req'd in number
symbol YES = 1					'LED display decimal point req'd in number
symbol ACTIVE = 2					'LED display decimal point is active (turned on)

symbol PRESSED = 1				'CPO routine if key/button pressed/closed
symbol ROGER = "R"				'Morse code "R" for "ROGER" to acknowedge Selector (S8) B-buttn pressed (accept command input)
symbol EOT = "+"					'prosign bar AR ("+") end-of-transmission marker

'__________________________________________________________________________________________________________________________
' Initial starting values (these are variable CAN be changed during program execution)
'__________________________________________________________________________________________________________________________
Read 0, NUMBER					'check EEPROM memory location 0 to see if it has previously saved program data variable

'255 value used to check if EEPORM has been initialized with default program data
if NUMBER <> 255 then				'251 used as indication EEPROM has previously saved program data (if not load default values)
	
	write 0, 255
	
	NUMBER = 15					'default beacon interval time in seconds
		write 1, NUMBER
	
	BCN_INTERVAL = NUMBER * 1000		'default beacon interval time in milliseconds (req'd for pause function)
	
	WPM = 20					'default wpm speed
		write 2, WPM
	
	GROUPS = 20					'default number of code groups to send
		write 3, GROUPS
		
else							'EEPROM has previously save program data so load them

	read 1, NUMBER				'EEPROM memory location 1 has beacon interval value in whole seconds
		BCN_INTERVAL = NUMBER * 1000	'convert to milliseconds for use with PICAXE pause
	
	read 2, WPM	
	read 3, GROUPS
	
end if

DIT = TC / WPM					'Morse code dit time in ms		

DASH = DIT * 3					'Morse code dash time in ms

DECIMAL = NO					'LED decimal point in number not req'd (default)

'__________________________________________________________________________________________________________________________
' Main program execution loops forever & get user menu selection & executes selected menu item

'	Variables: ITEM, SELECTOR, BTN_B
'	Calls: Display_Menu_Item, Read_Selector, Execute
'	Returns: Nothing
'__________________________________________________________________________________________________________________________
Main:
	
	'encoded alphanumeric values mapped to appropriate 7-segment LED segments to turn on/off
	'A = 119, b = 124, C = 57, c = 88, d = 94, E = 121, F = 113, G = 61, H = 118, i = 16, J = 30, L = 56
	'l = 48, N = 55, n = 84, O = 63, o = 92, P = 115, r = 80, S (5 is used) = 109, t = 120, U (also V) = 62, u = 28, y = 110, ? = 83

	ITEM = 1					'set default starting menu item 1 = Co (Morse code practice oscillator)

	do
		'encoded alphnumeric character req'd for LED display but ASCII numbers & upper case alpha characters req'd for Morse code
		'TX_Morse so all req'd variables by each routine are passed to Display_Menu_Item using four values
		select case ITEM			'determine menu item to display on 7-segment LED
			
			case 1			'display Co (57, 92) = Morse code practice oscillator		
				Display_Menu_Item(57, 92, "C", "O")
				
			case 2			'Al (119, 48) = practice Morse code alphabet A - Z (only)
				Display_Menu_Item(119, 56, "A", "L")
				
			case 3			'Nu (55, 28) = practice Morse code numbers 0 - 9 & punctuation / . , ? (only)
				Display_Menu_Item(55, 28, "N", "U")
							
			case 4			'An (119, 84) = practice Morse code combination of alphabet + numbers + punctuation
				Display_Menu_Item(119, 84, "A", "N")
			
			case 5 			'bE (124, 121) = beacon transmit (transmits Morse code info in Beacon lookup table)
				Display_Menu_Item(124, 121, "B", "E")
			
			case 6			'bi (94, 110) = beacon interval in milliseconds
				Display_Menu_Item(124, 16, "B", "I")
			
			case 7			'SP (109, 94)= word-per-minute speed
				Display_Menu_Item(109, 94, "S", "D")	
		
			case 8 			'Gr (61, 80) = number of Morse code practice groups to transmit
				Display_Menu_Item(61, 80, "G", "R")
		
			case 9			'Uo (62, 90) = get current PICAXE battery/power supply DC voltage (VPSU) (V is U on display)
				Display_Menu_Item(62, 92, "V", "O")
			
		endselect
	
		'1st item variable to receive new value, 2nd item minimum value, 3rd item maximum value
		Read_Selector(ITEM, 1, 10)	'read Selector (S8) A (C.5, pin 2), B (C.5, pin 2 + C.3, pin 4), C (C.3, pin 4)
				
		'run appropriate subroutine based on menu item selected
		if SELECTOR = BTN_B then
			
			call execute
		
		endif	
	
	loop

end

'__________________________________________________________________________________________________________________________
' Execute subroutine based on menu item selected

'	Variables: ITEM, MODULO, OFFSET, BCN_INTERVAL, NUMBER, WPM, DIT, TC, SELECTOR, BTN_B, GROUPS, VPSU
'	Calls: CPO, TX_Groups, TX_Beacon, Read_Selector, Display_Number 
'	Returns: WPM, DIT, BCN_INERVAL, GROUPS
'__________________________________________________________________________________________________________________________
Execute:

	'determine what subroutine to execute based on menu item selected
	select case ITEM				
		
		case 1				
			call CPO			'Co = code practice oscillator
		
		case 2				'Al = practice Morse code alphabet A to Z (only)
			MODULO = 26			'modulo to use for PRNG (for 26 letters)
			OFFSET = 14			'offset where alphabet starts in Get_Morse_Char lookup table
		
			call TX_Groups

		case 3				'Nu = practice Morse code numbers 0 to 9 & punctuation , ? . / (only)
			MODULO = 14			'for 14 numbers & punctuation
			OFFSET = 0
	
			call TX_Groups

		case 4				'An = practice Morse code combination of alphabet + numbers + punctuation
			MODULO = 40 		'for all 14 numbers & punctuation plus 26 letters
			OFFSET = 0
			
			call TX_Groups
			
		case 5				'bE = beacon transmit (transmits Morse code info stored in Beacon lookup table)
			do
				
				call Tx_Beacon	'transmit Morse code info stored in Beacon lookup table
			
				pause BCN_INTERVAL'wait for specific amount of time & repeat
				
			loop				
		
		case 6				'bi = beacon interval in milliseconds (ms) (min 0, max = 60000, 0 to 60 seconds)
			'convert to seconds & transfer to NUMBER variable
			NUMBER = BCN_INTERVAL / 1000
				FLAG = NUMBER	'used to determine if user changed value or not (saves on EEPROM write wear & tear)
				
			'display number & let user adjust value up/down until Selector B button pressed
			do	
				call Display_Number
				
				Read_Selector(NUMBER, 1, 60)		
	
			loop while SELECTOR <> BTN_B
			
			'write value to EEPROM if it has changed
			Write_EEPROM(FLAG, NUMBER, 1)
		
			'convert seconds back to milliseconds
			BCN_INTERVAL = NUMBER * 1000
		
		case 7				'SP = word-per-minute speed
			NUMBER = WPM		'transfer WPM to NUMBER variable
				FLAG = NUMBER
				
			'display number & let user adjust value up/down until Selector B button pressed
			do	
				call Display_Number
				
				Read_Selector(NUMBER, 5, 40)		
			
			loop while SELECTOR <> BTN_B
			
			Write_EEPROM(FLAG, NUMBER, 2)
				
			WPM = NUMBER		'transfer NUMBER to WPM variable
		
			DIT = TC / WPM		'recalculate Morse code dit time in ms
						
		case 8 				'Gr = number of Morse code practice groups to transmit
			NUMBER = GROUPS
				FLAG = NUMBER
				
			do	
				call Display_Number
				
				Read_Selector(NUMBER, 5, 99)		
			
			loop while SELECTOR <> BTN_B
		
			Write_EEPROM(FLAG, NUMBER, 3)
			
			GROUPS = NUMBER
	
		case 9				'Uo = get current PICAXE battery/power supply DC voltage (VDC)
			calibadc10 VPSU   	'resample 10-bit calibration & calculate VPSU to 10 mV resolution (with 5/4 rounding)
				VPSU = 52378 / VPSU * 2 + 6
		
			NUMBER = VPSU / 10	'convert to volts & tenths only
				
			DECIMAL = YES		'turn on LED display decimal point
			
			call Display_Number
			
			'display voltage until user presses Selector (S8) B button to exit routine
			do
		
				SELECTOR = pins and 40
					pause DEBOUNCE	
		
				if SELECTOR = BTN_B then
					
					CHARACTER = ROGER
					
					call TX_Morse
					
					exit
					
				endif
			
			loop
			
	endselect

return

'__________________________________________________________________________________________________________________________
' Morse code practice oscillator subroutine (must turn power off/on to reset)
	
'	Variables: DURATION, PRESS_SWITCH, PRESSED, RED_LED, SPEAKER, TONE
'	Calls: Nothing
'	Returns: Nothing
'__________________________________________________________________________________________________________________________
CPO:

	pullup %1000				'enable internal input pullup resistor on port c.4 (pin 3)

	do
  	
		'press switch used as Morse code hand key turns LED & speaker audio tone on and off
  		if PRESS_SWITCH = PRESSED then
    		
			'press_switch (hand key) is down, turn on red LED & produce speaker tone for 10 ms
    			high RED_LED
    		
			sound SPEAKER, (TONE, 2)
		
		endif
  
  		low RED_LED				'press switch is up (off) so also turn off red LED (speaker off by default)
	
	loop
	
return

'__________________________________________________________________________________________________________________________
' Transmit Morse code beacon alphabet characters using PRNG (must turn power off/on to reset

'	Variables: NUMBER, REPEATS, INDEX, CHARACTER, DASH, DIT, REPEATS, GROUPS, EOT
'	Calls: TX_Morse, Get_Random_Char
'	Returns: Nothing
'__________________________________________________________________________________________________________________________
TX_Groups:

	pause DASH					'wait before starting transmission
		
	NUMBER = 0
		REPEATS = 0
	
	for INDEX = 0 to 2			'send three V's to alert user that group transmission about to start
	
		CHARACTER = "V"
		
		call TX_Morse
	
	next INDEX
	
	do
		call Get_Random_Char		'get random alphanumeric character
		
		call TX_Morse			'transmit it
			
		inc NUMBER				'incrememt numer of alphanumeric characters transmitted so far
		
		if NUMBER = 5 then		'break into groups of five alphanumeric characters
			
			sertxd(" ")			'add space between groups & display on PICAXE serial terminal
			
			NUMBER = 0			
			
			CHARACTER = " "		'word spacing
			
			call TX_Morse
			
			inc REPEATS			'increment number of alphanumeric character groups of five transmitted so far
			
		endif
	
	loop while REPEATS < GROUPS		'once number of repeats = number of groups exit loop

	CHARACTER = EOT				'transmit prosign bar AR (end of transmission)
	
	call TX_Morse
	
return

'__________________________________________________________________________________________________________________________
' Generate pseudo-random Morse code alphanumeric character selected from lookup table to transmit on computer display
' MODULO & OFFSET values are set by calling routine to point to only numbers & punctuation, letter only & all characters

'	Variables: SEED, PRIKE_2, INDEX, MODULO, OFFSET, CHARACTER, TIME (system time in milliseconds 0 to 65536 & recycles)
'	Calls: Nothing
'	Returns: CHARACTER
'__________________________________________________________________________________________________________________________
Get_Random_Char:

	random SEED					'generate pseudo random number
	
	'increase pseudo randomness using system time & Mersenne prime number
	SEED = SEED XOR TIME XOR PRIME_2	
	
	INDEX = SEED // MODULO			'get pseudo random number using modulo value req'd (// or & is modulo function)
	
	INDEX = INDEX + OFFSET			'add in req'd offset for lookup table
		
	lookup INDEX, ("0123456789,?./ABCDEFGHIJKLMNOPQRSTUVWXYZ"), CHARACTER

	sertxd(CHARACTER)				'display alphanumeric characters on PICAXE terminal display
	
return

'__________________________________________________________________________________________________________________________
' Transmit Morse code beacon using data stored in Beacon lookup table & keys Q1 to optionally key transmitter

'	Variables: DELAY, INDEX, CHARACTER, INTERVAL, DIT, RED_LED
'	Calls: Beacon, Tx_Morse
'	Returns: Nothing
'__________________________________________________________________________________________________________________________
Tx_Beacon:
	
	'because LED & LED MC display shared, tx will be keyed with S8 selector button presses - press button B & connect Q1 ground to tx
	pause DELAY					'connect transmitter ground keying line to Q2 (avoids keying tx during menu setup/select)

	INDEX = 0					'reset lookup table index pointer
	
	do
		
		'get next CW character from lookup table	
		Beacon(INDEX, CHARACTER)		
		
		if CHARACTER = EOT then		'end of Beacon then send end-of-transmission character (EOT = "+")
			
			call TX_Morse		'transmit encoded Morse character (decoded on-the-fly) & exit loop
		
			exit	
		
		endif
	
		sertxd(CHARACTER)			'display alphanumeric characters on PICAXE terminal display		
		
		call TX_Morse			'transmit encoded Morse character (decoded on the fly)
			
		inc INDEX				'next character
	
	loop

return

'__________________________________________________________________________________________________________________________
' Routine to decode & transmit encoded Morse characters using bit shifting 
' Based on original coded by Steve Paige, VK6HV (with slight modifications by Robert, VA3ROM)

' RED_LED also turns Q1 on/off for key down/key up any connected CW transmitter

' Morse code encoded data table converts from ASCII to binary tree encoded values

'	Variables: CHARACTER, ENCODED, DASH, DIT, RED_LED, FLAG, DURATION, SPEAKER, TONE 
'	Calls: Nothing
'	Returns: Nothing
'__________________________________________________________________________________________________________________________
TX_Morse:
	
	'check for word space character first (saves time)
	if CHARACTER = " " then
		
		DASH = DIT * 7			'standard is DIT * 7
			pause DASH
	
		return
	
	endif			

	DASH = DIT * 3				'default equates to dash (dah)
	
	'encode character for tx (converts from ASCII to specific Morse code encoded value)
	CHARACTER = CHARACTER - 32		'convert from ASCII
			
	'search for Morse code encoded value in lookup table
	lookup CHARACTER, (0,117,82,0,200,0,34,94,45,109,0,42,115,97,106,41,63,62,60,56,48,32,33,35,39,47,71,85,0,_
				49,0,76,86,6,17,21,9,2,20,11,16,4,30,13,18,7,5,15,22,27,10,8,3,12,24,14,25,29,19), ENCODED
	
	'right bit shift (division) routine keeps going until Morse code character shifted value equals 1
	do

		high RED_LED			'turn on red LED (also turns on Q1 & key down any connected CW transmiter)
		
		FLAG = ENCODED // 2		'modulo function determines if encoded number value odd or even (1 or 0)
		
		if FLAG = 1 then			'odd so do dah
		
			'conversion req'd for speaker interval in 10 ms increments (this is why TC variable is set faster than 1200 ms)
			DURATION = DASH / 10
				sound SPEAKER, (TONE, DURATION)
  	
		else					'even so do dit
					
			'conversion req'd for speaker interval in 10 ms increments (this is why TC variable is set faster than 1200 ms)
			DURATION = DIT / 10
				sound SPEAKER, (TONE, DURATION)
			
		endif
				
		low RED_LED				'turn off red LED (also turns off Q1 & key up any connected CW transmiter)
		
		ENCODED = ENCODED / 2	      'division by 2 performs binary right bit shift
		
		if ENCODED <> 1 then		'add inter-element spacing (if req'd)
			
			pause DIT
			
		endif
		
	loop until ENCODED = 1			'loop until all bits shifted & value of 1 reached

	'DASH = DIT * 3			'add character spacing
	pause DASH
		
return			

'__________________________________________________________________________________________________________________________
' Display encoded character in right or leftmost 7-segment LED (2 x C4094B shift-and-store bus registers)

'	Variables: BIT_MASK, SEGMENT, ENCODED (holds encoded character value for 7-segment LED display)
'	Calls: Nothing
'	Returns: Nothing
'__________________________________________________________________________________________________________________________
Display_Character: 
	
	BIT_MASK = 1				'start at first bit in encoded CHARACTER variable

	do
		'check bit in encode CHARACTER for 0 or 1 to determine if segment should be turned on or off
		SEGMENT = ENCODED and BIT_MASK			
	
		if SEGMENT > 0 then		'should this LED segment be turned on or off?
			low S_DATA			'it's 0 turn LED segment off
		else 	
			high S_DATA		 	'it's 1 turn LED legment on
		end if
	
		'pulse clock pin to load status of LED segment (2 x C4094B shift-and-store bus registers)
		high W_CLOCK			
			low W_CLOCK			 		
					
		BIT_MASK = BIT_MASK * 2		'left bit shift & point to next LED segment to check
	
	'left bit shift equals 256 exceeding max. bit value 255, which wraps around to 0 & exits loop/subroutine
	loop while BIT_MASK <> 0	

	'low RED_LED
	
return

'__________________________________________________________________________________________________________________________
' Display individual digits of number & displays each shifting from rightmost then leftmost LED

' Note: PICACE 08M2+ has 128 byte of extra user accessible memory registers using peek (get from) & poke (save to) commands

'	Variables: INDEX, NUMBER, DECIMAL
'	Calls: Get_Number
'	Returns: Nothing
'__________________________________________________________________________________________________________________________
Display_Number: 

	DIT = TC / WPM_MENU			'send menu numbers @ default 20 wpm (in case user has really slow speed)
	
	INDEX = NUMBER / 10			'leftmost digit to display (0 to 9)			
		poke 31, INDEX			'save 1st digit to memory register 31 (ran out of regular PICAXE vars so use scratch memory)
	
	call Get_Number				'get & display number (CHARACTER)				 
	
	INDEX = NUMBER // 10			'rightmost digit to display (0 to 9)
		poke 32, INDEX			'save 2nd digit to memory register 32
			
	call Get_Number				'get & display number (CHARACTER)
	
	peek 31, CHARACTER			'get 1st digit from memory register 31
		CHARACTER = CHARACTER + 48	'convert to ASCII
	
	call TX_Morse				'transmit number in Morse code
	
	'pause DASH					'pause for character spacing
	
	if DECIMAL = ACTIVE then		'LED decimal point active
		
		CHARACTER = "."			'transmit Morse code decimal point (period)
			call TX_Morse
		
		'pause DASH				'pause for character spacing
			DECIMAL = NO		'reset LED decimal flag (not req'd for 2nd number)
	
	endif

	peek 32, CHARACTER			'retrieve 2nd digit from memory register 32
		CHARACTER = CHARACTER + 48	'convert to ASCII
		
	call TX_Morse				'transmit number in Morse code
		
	DIT = TC / WPM				'restore user wpm speed 
	
return

'__________________________________________________________________________________________________________________________
' Lookup encoded value for a single digit number 0 to 9 & display on right or leftmost LED

'	Variables: INDEX, CHARACTER
'	Calls: Display_Character
'	Returns: CHARACTER
'__________________________________________________________________________________________________________________________
Get_Number:
		
	'find 7-segment LED numeric patterns to turn on/off represented

	'0 = 63, 1 = 6, 2 = 91, 3 = 79, 4 = 102, 5 = 109, 6 = 125, 7 = 7, 8 = 127, 9 = 111
	lookup INDEX, (63,6,91,79,102,109,125,7,127,111), ENCODED
	
	if DECIMAL = YES then			'LED decimal point req'd
		
		ENCODED = ENCODED + 128		'set LED decimal point with this number
		
		DECIMAL = ACTIVE			'LED dicmal point active
	
	endif
	
	call Display_Character
			
return
